# coding: utf-8

# flake8: noqa
"""
    Rapidata.Dataset

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: v1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


# import models into model package
from rapidata.api_client.models.add_validation_rapid_model import AddValidationRapidModel
from rapidata.api_client.models.add_validation_rapid_model_payload import AddValidationRapidModelPayload
from rapidata.api_client.models.add_validation_rapid_model_truth import AddValidationRapidModelTruth
from rapidata.api_client.models.add_validation_rapid_result import AddValidationRapidResult
from rapidata.api_client.models.admin_order_model import AdminOrderModel
from rapidata.api_client.models.admin_order_model_paged_result import AdminOrderModelPagedResult
from rapidata.api_client.models.age_group import AgeGroup
from rapidata.api_client.models.age_user_filter_model import AgeUserFilterModel
from rapidata.api_client.models.aggregator_type import AggregatorType
from rapidata.api_client.models.attach_category_rapid_blueprint import AttachCategoryRapidBlueprint
from rapidata.api_client.models.attach_category_result import AttachCategoryResult
from rapidata.api_client.models.attach_category_truth import AttachCategoryTruth
from rapidata.api_client.models.base_error import BaseError
from rapidata.api_client.models.bounding_box_payload import BoundingBoxPayload
from rapidata.api_client.models.bounding_box_rapid_blueprint import BoundingBoxRapidBlueprint
from rapidata.api_client.models.bounding_box_result import BoundingBoxResult
from rapidata.api_client.models.bounding_box_truth import BoundingBoxTruth
from rapidata.api_client.models.box_shape import BoxShape
from rapidata.api_client.models.classification_metadata import ClassificationMetadata
from rapidata.api_client.models.classification_metadata_filter_config import ClassificationMetadataFilterConfig
from rapidata.api_client.models.classify_payload import ClassifyPayload
from rapidata.api_client.models.clone_dataset_model import CloneDatasetModel
from rapidata.api_client.models.clone_order_model import CloneOrderModel
from rapidata.api_client.models.clone_order_result import CloneOrderResult
from rapidata.api_client.models.compare_payload import ComparePayload
from rapidata.api_client.models.compare_rapid_blueprint import CompareRapidBlueprint
from rapidata.api_client.models.compare_result import CompareResult
from rapidata.api_client.models.compare_truth import CompareTruth
from rapidata.api_client.models.compare_workflow_config import CompareWorkflowConfig
from rapidata.api_client.models.compare_workflow_config_rapid_selection_configs_inner import CompareWorkflowConfigRapidSelectionConfigsInner
from rapidata.api_client.models.compare_workflow_config_referee import CompareWorkflowConfigReferee
from rapidata.api_client.models.compare_workflow_model import CompareWorkflowModel
from rapidata.api_client.models.conditional_validation_rapid_selection_config import ConditionalValidationRapidSelectionConfig
from rapidata.api_client.models.coordinate import Coordinate
from rapidata.api_client.models.correlated_rapid_selection_config import CorrelatedRapidSelectionConfig
from rapidata.api_client.models.count_classification_metadata_filter_config import CountClassificationMetadataFilterConfig
from rapidata.api_client.models.count_metadata import CountMetadata
from rapidata.api_client.models.country_user_filter_model import CountryUserFilterModel
from rapidata.api_client.models.create_client_model import CreateClientModel
from rapidata.api_client.models.create_complex_order_model import CreateComplexOrderModel
from rapidata.api_client.models.create_complex_order_model_pipeline import CreateComplexOrderModelPipeline
from rapidata.api_client.models.create_complex_order_result import CreateComplexOrderResult
from rapidata.api_client.models.create_dataset_artifact_model import CreateDatasetArtifactModel
from rapidata.api_client.models.create_dataset_artifact_model_dataset import CreateDatasetArtifactModelDataset
from rapidata.api_client.models.create_default_order_model import CreateDefaultOrderModel
from rapidata.api_client.models.create_default_order_model_workflow_config import CreateDefaultOrderModelWorkflowConfig
from rapidata.api_client.models.create_demographic_rapid_model import CreateDemographicRapidModel
from rapidata.api_client.models.create_empty_validation_set_result import CreateEmptyValidationSetResult
from rapidata.api_client.models.create_legacy_order_result import CreateLegacyOrderResult
from rapidata.api_client.models.create_order_model import CreateOrderModel
from rapidata.api_client.models.create_order_model_referee import CreateOrderModelReferee
from rapidata.api_client.models.create_order_model_selections_inner import CreateOrderModelSelectionsInner
from rapidata.api_client.models.create_order_model_user_filters_inner import CreateOrderModelUserFiltersInner
from rapidata.api_client.models.create_order_model_workflow import CreateOrderModelWorkflow
from rapidata.api_client.models.create_order_result import CreateOrderResult
from rapidata.api_client.models.create_simple_pipeline_model import CreateSimplePipelineModel
from rapidata.api_client.models.create_simple_pipeline_model_artifacts_inner import CreateSimplePipelineModelArtifactsInner
from rapidata.api_client.models.create_simple_pipeline_model_pipeline_steps_inner import CreateSimplePipelineModelPipelineStepsInner
from rapidata.api_client.models.create_unsupported_order_model import CreateUnsupportedOrderModel
from rapidata.api_client.models.customer_order_model import CustomerOrderModel
from rapidata.api_client.models.customer_order_model_paged_result import CustomerOrderModelPagedResult
from rapidata.api_client.models.datapoint import Datapoint
from rapidata.api_client.models.datapoint_asset import DatapointAsset
from rapidata.api_client.models.datapoint_get_by_id_get200_response import DatapointGetByIdGet200Response
from rapidata.api_client.models.datapoint_metadata_model import DatapointMetadataModel
from rapidata.api_client.models.datapoint_metadata_model_metadata_inner import DatapointMetadataModelMetadataInner
from rapidata.api_client.models.datapoint_model import DatapointModel
from rapidata.api_client.models.datapoint_model_asset import DatapointModelAsset
from rapidata.api_client.models.dataset_evaluation_step_model import DatasetEvaluationStepModel
from rapidata.api_client.models.demographic_rapid_selection_config import DemographicRapidSelectionConfig
from rapidata.api_client.models.demographic_selection import DemographicSelection
from rapidata.api_client.models.early_stopping_referee_model import EarlyStoppingRefereeModel
from rapidata.api_client.models.empty_validation_truth import EmptyValidationTruth
from rapidata.api_client.models.error_type import ErrorType
from rapidata.api_client.models.feature_flag import FeatureFlag
from rapidata.api_client.models.feature_flag_model import FeatureFlagModel
from rapidata.api_client.models.feedback_model import FeedbackModel
from rapidata.api_client.models.file_asset import FileAsset
from rapidata.api_client.models.file_asset_metadata_inner import FileAssetMetadataInner
from rapidata.api_client.models.file_asset_model import FileAssetModel
from rapidata.api_client.models.filter import Filter
from rapidata.api_client.models.filter_operator import FilterOperator
from rapidata.api_client.models.free_text_payload import FreeTextPayload
from rapidata.api_client.models.free_text_rapid_blueprint import FreeTextRapidBlueprint
from rapidata.api_client.models.free_text_result import FreeTextResult
from rapidata.api_client.models.gender import Gender
from rapidata.api_client.models.gender_user_filter_model import GenderUserFilterModel
from rapidata.api_client.models.get_available_validation_sets_result import GetAvailableValidationSetsResult
from rapidata.api_client.models.get_datapoints_by_dataset_id_result import GetDatapointsByDatasetIdResult
from rapidata.api_client.models.get_dataset_by_id_result import GetDatasetByIdResult
from rapidata.api_client.models.get_order_by_id_result import GetOrderByIdResult
from rapidata.api_client.models.get_order_results_result import GetOrderResultsResult
from rapidata.api_client.models.get_public_orders_result import GetPublicOrdersResult
from rapidata.api_client.models.get_workflow_config_result import GetWorkflowConfigResult
from rapidata.api_client.models.get_workflow_config_result_workflow_config import GetWorkflowConfigResultWorkflowConfig
from rapidata.api_client.models.image_dimension_metadata import ImageDimensionMetadata
from rapidata.api_client.models.import_from_file_result import ImportFromFileResult
from rapidata.api_client.models.import_validation_set_from_file_result import ImportValidationSetFromFileResult
from rapidata.api_client.models.issue_auth_token_result import IssueAuthTokenResult
from rapidata.api_client.models.issue_client_auth_token_result import IssueClientAuthTokenResult
from rapidata.api_client.models.labeling_selection import LabelingSelection
from rapidata.api_client.models.line import Line
from rapidata.api_client.models.line_payload import LinePayload
from rapidata.api_client.models.line_point import LinePoint
from rapidata.api_client.models.line_rapid_blueprint import LineRapidBlueprint
from rapidata.api_client.models.line_result import LineResult
from rapidata.api_client.models.line_truth import LineTruth
from rapidata.api_client.models.locate_box_truth import LocateBoxTruth
from rapidata.api_client.models.locate_coordinate import LocateCoordinate
from rapidata.api_client.models.locate_payload import LocatePayload
from rapidata.api_client.models.locate_rapid_blueprint import LocateRapidBlueprint
from rapidata.api_client.models.locate_result import LocateResult
from rapidata.api_client.models.location_metadata import LocationMetadata
from rapidata.api_client.models.location_metadata_exists_filter_config import LocationMetadataExistsFilterConfig
from rapidata.api_client.models.logic_operator import LogicOperator
from rapidata.api_client.models.login_model import LoginModel
from rapidata.api_client.models.metadata_visibilities import MetadataVisibilities
from rapidata.api_client.models.multi_asset import MultiAsset
from rapidata.api_client.models.multi_asset_model import MultiAssetModel
from rapidata.api_client.models.naive_referee_config import NaiveRefereeConfig
from rapidata.api_client.models.naive_referee_model import NaiveRefereeModel
from rapidata.api_client.models.named_classification import NamedClassification
from rapidata.api_client.models.named_entity_payload import NamedEntityPayload
from rapidata.api_client.models.named_entity_rapid_blueprint import NamedEntityRapidBlueprint
from rapidata.api_client.models.named_entity_result import NamedEntityResult
from rapidata.api_client.models.named_entity_truth import NamedEntityTruth
from rapidata.api_client.models.never_ending_referee_config import NeverEndingRefereeConfig
from rapidata.api_client.models.newsletter_model import NewsletterModel
from rapidata.api_client.models.no_validation_workflow_rapid_selection_config import NoValidationWorkflowRapidSelectionConfig
from rapidata.api_client.models.null_asset import NullAsset
from rapidata.api_client.models.null_asset_model import NullAssetModel
from rapidata.api_client.models.only_validation_workflow_rapid_selection_config import OnlyValidationWorkflowRapidSelectionConfig
from rapidata.api_client.models.order_model import OrderModel
from rapidata.api_client.models.order_query_get200_response import OrderQueryGet200Response
from rapidata.api_client.models.original_filename_metadata import OriginalFilenameMetadata
from rapidata.api_client.models.page_info import PageInfo
from rapidata.api_client.models.polygon_payload import PolygonPayload
from rapidata.api_client.models.polygon_rapid_blueprint import PolygonRapidBlueprint
from rapidata.api_client.models.polygon_result import PolygonResult
from rapidata.api_client.models.polygon_truth import PolygonTruth
from rapidata.api_client.models.private_text_metadata_input import PrivateTextMetadataInput
from rapidata.api_client.models.probabilistic_attach_category_referee_config import ProbabilisticAttachCategoryRefereeConfig
from rapidata.api_client.models.prompt_metadata import PromptMetadata
from rapidata.api_client.models.prompt_metadata_input import PromptMetadataInput
from rapidata.api_client.models.public_text_metadata_input import PublicTextMetadataInput
from rapidata.api_client.models.query_model import QueryModel
from rapidata.api_client.models.query_orders_model import QueryOrdersModel
from rapidata.api_client.models.query_validation_rapids_result import QueryValidationRapidsResult
from rapidata.api_client.models.query_validation_rapids_result_asset import QueryValidationRapidsResultAsset
from rapidata.api_client.models.query_validation_rapids_result_paged_result import QueryValidationRapidsResultPagedResult
from rapidata.api_client.models.query_validation_set_model import QueryValidationSetModel
from rapidata.api_client.models.rapid_result_model import RapidResultModel
from rapidata.api_client.models.rapid_result_model_result import RapidResultModelResult
from rapidata.api_client.models.rapid_skipped_model import RapidSkippedModel
from rapidata.api_client.models.request_password_reset_command import RequestPasswordResetCommand
from rapidata.api_client.models.root_filter import RootFilter
from rapidata.api_client.models.send_completion_mail_step_model import SendCompletionMailStepModel
from rapidata.api_client.models.shape import Shape
from rapidata.api_client.models.signup_customer_model import SignupCustomerModel
from rapidata.api_client.models.signup_shadow_customer_model import SignupShadowCustomerModel
from rapidata.api_client.models.simple_workflow_config import SimpleWorkflowConfig
from rapidata.api_client.models.simple_workflow_config_blueprint import SimpleWorkflowConfigBlueprint
from rapidata.api_client.models.simple_workflow_model import SimpleWorkflowModel
from rapidata.api_client.models.simple_workflow_model_blueprint import SimpleWorkflowModelBlueprint
from rapidata.api_client.models.skip_result import SkipResult
from rapidata.api_client.models.sort_criterion import SortCriterion
from rapidata.api_client.models.sort_direction import SortDirection
from rapidata.api_client.models.static_rapid_selection_config import StaticRapidSelectionConfig
from rapidata.api_client.models.submit_coco_model import SubmitCocoModel
from rapidata.api_client.models.submit_coco_result import SubmitCocoResult
from rapidata.api_client.models.submit_password_reset_command import SubmitPasswordResetCommand
from rapidata.api_client.models.text_asset import TextAsset
from rapidata.api_client.models.text_asset_model import TextAssetModel
from rapidata.api_client.models.text_metadata import TextMetadata
from rapidata.api_client.models.transcription_metadata import TranscriptionMetadata
from rapidata.api_client.models.transcription_metadata_input import TranscriptionMetadataInput
from rapidata.api_client.models.transcription_payload import TranscriptionPayload
from rapidata.api_client.models.transcription_rapid_blueprint import TranscriptionRapidBlueprint
from rapidata.api_client.models.transcription_result import TranscriptionResult
from rapidata.api_client.models.transcription_truth import TranscriptionTruth
from rapidata.api_client.models.transcription_word import TranscriptionWord
from rapidata.api_client.models.unlock_order_result import UnlockOrderResult
from rapidata.api_client.models.update_access_model import UpdateAccessModel
from rapidata.api_client.models.update_order_model import UpdateOrderModel
from rapidata.api_client.models.update_workflow_config_model import UpdateWorkflowConfigModel
from rapidata.api_client.models.update_workflow_config_model_workflow_config import UpdateWorkflowConfigModelWorkflowConfig
from rapidata.api_client.models.upload_coco_result import UploadCocoResult
from rapidata.api_client.models.upload_datapoints_result import UploadDatapointsResult
from rapidata.api_client.models.upload_files_from_s3_bucket_model import UploadFilesFromS3BucketModel
from rapidata.api_client.models.upload_text_sources_to_dataset_model import UploadTextSourcesToDatasetModel
from rapidata.api_client.models.validation_chance import ValidationChance
from rapidata.api_client.models.validation_import_post_request_blueprint import ValidationImportPostRequestBlueprint
from rapidata.api_client.models.validation_rapid_selection_config import ValidationRapidSelectionConfig
from rapidata.api_client.models.validation_selection import ValidationSelection
from rapidata.api_client.models.validation_set import ValidationSet
from rapidata.api_client.models.validation_set_overview_model import ValidationSetOverviewModel
from rapidata.api_client.models.validation_set_paged_result import ValidationSetPagedResult
from rapidata.api_client.models.workflow_aggregation_step_model import WorkflowAggregationStepModel
from rapidata.api_client.models.workflow_labeling_step_model import WorkflowLabelingStepModel
from rapidata.api_client.models.workflow_split_model import WorkflowSplitModel
from rapidata.api_client.models.workflow_split_model_filter_configs_inner import WorkflowSplitModelFilterConfigsInner



from itertools import zip_longest
import os

from rapidata.api_client.models.datapoint_metadata_model import DatapointMetadataModel
from rapidata.api_client.models.datapoint_metadata_model_metadata_inner import (
    DatapointMetadataModelMetadataInner,
)
from rapidata.api_client.models.upload_text_sources_to_dataset_model import (
    UploadTextSourcesToDatasetModel,
)
from rapidata.rapidata_client.metadata.base_metadata import Metadata
from rapidata.service import LocalFileService
from rapidata.service.openapi_service import OpenAPIService
from concurrent.futures import ThreadPoolExecutor, as_completed
from tqdm import tqdm


class RapidataDataset:

    def __init__(self, dataset_id: str, openapi_service: OpenAPIService):
        self.dataset_id = dataset_id
        self.openapi_service = openapi_service
        self.local_file_service = LocalFileService()

    def add_texts(self, texts: list[str]):
        model = UploadTextSourcesToDatasetModel(
            datasetId=self.dataset_id, textSources=texts
        )
        self.openapi_service.dataset_api.dataset_upload_text_sources_to_dataset_post(
            model
        )

    def add_media_from_paths(
        self,
        image_paths: list[str | list[str]],
        metadata: list[Metadata] | None = None,
        max_workers: int = 10,
    ):
        if metadata is not None and len(metadata) != len(image_paths):
            raise ValueError(
                "metadata must be None or have the same length as image_paths"
            )

        def upload_datapoint(media_paths_rapid: str | list[str], meta: Metadata | None) -> None:
            if isinstance(media_paths_rapid, list) and not all(
                os.path.exists(media_path) for media_path in media_paths_rapid
            ):
                raise FileNotFoundError(f"File not found: {media_paths_rapid}")
            elif isinstance(media_paths_rapid, str) and not os.path.exists(
                media_paths_rapid
            ):
                raise FileNotFoundError(f"File not found: {media_paths_rapid}")

            meta_model = meta.to_model() if meta else None
            model = DatapointMetadataModel(
                datasetId=self.dataset_id,
                metadata=(
                    [DatapointMetadataModelMetadataInner(meta_model)]
                    if meta_model
                    else []
                ),
            )

            self.openapi_service.dataset_api.dataset_create_datapoint_post(
                model=model,
                files=media_paths_rapid if isinstance(media_paths_rapid, list) else [media_paths_rapid] # type: ignore
            )

        total_uploads = len(image_paths)
        with ThreadPoolExecutor(max_workers=max_workers) as executor:
            futures = [
                executor.submit(upload_datapoint, media_paths, meta)
                for media_paths, meta in zip_longest(image_paths, metadata or [])
            ]

            with tqdm(total=total_uploads, desc="Uploading datapoints") as pbar:
                for future in as_completed(futures):
                    future.result()  # This will raise any exceptions that occurred during execution
                    pbar.update(1)



from rapidata.api_client.api.dataset_api import DatasetApi
from rapidata.api_client.api.identity_api import IdentityApi
from rapidata.api_client.api.order_api import OrderApi
from rapidata.api_client.api.rapid_api import RapidApi
from rapidata.api_client.api.validation_api import ValidationApi
from rapidata.api_client.api_client import ApiClient
from rapidata.api_client.configuration import Configuration


class OpenAPIService:

    def __init__(
        self,
        client_id: str,
        client_secret: str,
        endpoint: str = "https://api.rapidata.ai",
    ):
        client_configuration = Configuration(host=endpoint)
        self.api_client = ApiClient(configuration=client_configuration)

        identity_api = IdentityApi(self.api_client)

        result = identity_api.identity_get_client_auth_token_post(
            client_id=client_id,
            _headers={"Authorization": f"Basic {client_secret}"},
        )

        client_configuration.api_key["bearer"] = f"Bearer {result.auth_token}"
        self._api_client = ApiClient()
        self._order_api = OrderApi(self.api_client)
        self._dataset_api = DatasetApi(self.api_client)

    @property
    def order_api(self) -> OrderApi:
        return self._order_api

    @property
    def dataset_api(self) -> DatasetApi:
        return self._dataset_api

    @property
    def validation_api(self) -> ValidationApi:
        return ValidationApi(self.api_client)

    @property
    def rapid_api(self) -> RapidApi:
        return RapidApi(self.api_client)



import time
from rapidata.rapidata_client.dataset.rapidata_dataset import RapidataDataset
from rapidata.service.openapi_service import OpenAPIService


class RapidataOrder:
    """
    Represents a Rapidata order.

    :param name: The name of the order.
    :type name: str
    :param workflow: The workflow associated with the order.
    :type workflow: Workflow
    :param rapidata_service: The Rapidata service used to create and manage the order.
    :type rapidata_service: RapidataService
    """

    def __init__(
        self,
        order_id: str,
        dataset: RapidataDataset,
        openapi_service: OpenAPIService,
    ):
        self.openapi_service = openapi_service
        self.order_id = order_id
        self._dataset = dataset

    def submit(self):
        """
        Submits the order for processing.
        """

        self.openapi_service.order_api.order_submit_post(self.order_id)

    def approve(self):
        """
        Approves the order for execution.
        """
        self.openapi_service.order_api.order_approve_post(self.order_id)

    def get_status(self):
        """
        Gets the status of the order.

        :return: The status of the order.
        :rtype: str
        """
        return self.openapi_service.order_api.order_get_by_id_get(self.order_id)

    def wait_for_done(self):
        """
        Blocking call that waits for the order to be done. Exponential backoff is used to check the status of the order.
        """
        wait_time = 2
        back_off_factor = 1.5
        minimum_poll_interval = 60 * 10  # 10 minutes

        while True:
            time.sleep(wait_time)
            result = self.get_status()
            if result.state == "Completed":
                break
            wait_time = max(
                minimum_poll_interval, wait_time * back_off_factor
            )  # poll at least every 10 minutes

    def get_results(self):
        """
        Gets the results of the order.

        :return: The results of the order.
        :rtype: dict
        """
        # return self.openapi_service.order_api.order_get_order_results_get(self.order_id) # throws error
        raise NotImplementedError("Currently not supported")

    @property
    def dataset(self):
        """
        The dataset associated with the order.
        :return: The RapidataDataset instance.
        :rtype: RapidataDataset
        """
        return self._dataset



from dataclasses import dataclass
from typing import Any

from rapidata.api_client.models.attach_category_truth import AttachCategoryTruth
from rapidata.api_client.models.bounding_box_payload import BoundingBoxPayload
from rapidata.api_client.models.bounding_box_truth import BoundingBoxTruth
from rapidata.api_client.models.classify_payload import ClassifyPayload
from rapidata.api_client.models.compare_payload import ComparePayload
from rapidata.api_client.models.compare_truth import CompareTruth
from rapidata.api_client.models.empty_validation_truth import EmptyValidationTruth
from rapidata.api_client.models.free_text_payload import FreeTextPayload
from rapidata.api_client.models.line_payload import LinePayload
from rapidata.api_client.models.line_truth import LineTruth
from rapidata.api_client.models.locate_box_truth import LocateBoxTruth
from rapidata.api_client.models.locate_payload import LocatePayload
from rapidata.api_client.models.named_entity_payload import NamedEntityPayload
from rapidata.api_client.models.named_entity_truth import NamedEntityTruth
from rapidata.api_client.models.polygon_payload import PolygonPayload
from rapidata.api_client.models.polygon_truth import PolygonTruth
from rapidata.api_client.models.transcription_payload import TranscriptionPayload
from rapidata.api_client.models.transcription_truth import TranscriptionTruth
from rapidata.rapidata_client.metadata.base_metadata import Metadata


@dataclass
class ValidatioRapidParts:
    question: str
    media_paths: str | list[str]
    payload: (
        BoundingBoxPayload
        | ClassifyPayload
        | ComparePayload
        | FreeTextPayload
        | LinePayload
        | LocatePayload
        | NamedEntityPayload
        | PolygonPayload
        | TranscriptionPayload
    )
    truths: (
        AttachCategoryTruth
        | BoundingBoxTruth
        | CompareTruth
        | EmptyValidationTruth
        | LineTruth
        | LocateBoxTruth
        | NamedEntityTruth
        | PolygonTruth
        | TranscriptionTruth
    )
    metadata: list[Metadata]
    randomCorrectProbability: float



from rapidata.api_client.models.classify_payload import ClassifyPayload
from rapidata.api_client.models.create_demographic_rapid_model import (
    CreateDemographicRapidModel,
)
from rapidata.service.openapi_service import OpenAPIService


class Utils:
    def __init__(self, openapi_service: OpenAPIService):
        self.openapi_service = openapi_service

    def new_demographic_rapid(
        self, question: str, options: list[str], identifier: str, media_path: str
    ):
        payload = ClassifyPayload(
            _t="ClassifyPayload", title=question, possibleCategories=options
        )
        model = CreateDemographicRapidModel(identifier=identifier, payload=payload)

        self.openapi_service.rapid_api.rapid_create_demographic_rapid_post(
            model=model, file=[media_path]
        )



from rapidata.rapidata_client.dataset.rapidata_validation_set import (
    RapidataValidationSet,
)
from rapidata.rapidata_client.dataset.validation_set_builder import ValidationSetBuilder
from rapidata.rapidata_client.order.rapidata_order_builder import RapidataOrderBuilder
from rapidata.rapidata_client.utils.utils import Utils
from rapidata.service.openapi_service import OpenAPIService


class RapidataClient:
    """The Rapidata client is the main entry point for interacting with the Rapidata API. It allows you to create orders and validation sets. For creating a new order, check out `new_order()`. For creating a new validation set, check out `new_validation_set()`."""

    def __init__(
        self,
        client_id: str,
        client_secret: str,
        endpoint: str = "https://api.app.rapidata.ai",
    ):
        """Initialize the RapidataClient. Best practice is to store the client ID and client secret in environment variables. Ask your Rapidata representative for the client ID and client secret.

        Args:
            client_id (str): The client ID for authentication.
            client_secret (str): The client secret for authentication.
            endpoint (str, optional): The API endpoint URL.
        """
        self.openapi_service = OpenAPIService(
            client_id=client_id, client_secret=client_secret, endpoint=endpoint
        )

    def new_order(self, name: str) -> RapidataOrderBuilder:
        """Create a new order using a RapidataOrderBuilder instance.

        Args:
            name (str): The name of the order.

        Returns:
            RapidataOrderBuilder: A RapidataOrderBuilder instance.
        """
        return RapidataOrderBuilder(openapi_service=self.openapi_service, name=name)

    def new_validation_set(self, name: str) -> ValidationSetBuilder:
        """Create a new validation set using a ValidationDatasetBuilder instance.

        Args:
            name (str): The name of the validation set.

        Returns:
            ValidationSetBuilder: A ValidationDatasetBuilder instance.
        """
        return ValidationSetBuilder(name=name, openapi_service=self.openapi_service)

    def get_validation_set(self, validation_set_id: str) -> RapidataValidationSet:
        """Get a validation set by ID.

        Args:
            validation_set_id (str): The ID of the validation set.

        Returns:
            RapidataValidationSet: The ValidationSet instance.
        """
        return RapidataValidationSet(validation_set_id, self.openapi_service)

    @property
    def utils(self) -> Utils:
        """Get the Utils instance.

        Returns:
            Utils: The Utils instance associated with this client.
        """
        return Utils(openapi_service=self.openapi_service)



from typing import Any
from rapidata.rapidata_client.referee.base_referee import Referee
from rapidata.api_client.models.early_stopping_referee_model import (
    EarlyStoppingRefereeModel,
)


class ClassifyEarlyStoppingReferee(Referee):
    """A referee that stops the task when confidence in the winning category exceeds a threshold.

    This referee implements an early stopping mechanism for classification tasks.
    It terminates the task when the confidence in the leading category surpasses
    a specified threshold or when the maximum number of votes is reached.

    The threshold behaves logarithmically, meaning small increments (e.g., from 0.99
    to 0.999) can significantly impact the stopping criteria.

    Attributes:
        threshold (float): The confidence threshold for early stopping.
        max_vote_count (int): The maximum number of votes allowed before stopping.
    """

    def __init__(self, threshold: float = 0.999, max_vote_count: int = 100):
        """Initialize the ClassifyEarlyStoppingReferee.

        Args:
            threshold (float, optional): The confidence threshold for early stopping.
                Defaults to 0.999.
            max_vote_count (int, optional): The maximum number of votes allowed
                before stopping. Defaults to 100.
        """
        self.threshold = threshold
        self.max_vote_count = max_vote_count

    def to_dict(self):
        return {
            "_t": "ProbabilisticAttachCategoryRefereeConfig",
            "threshold": self.threshold,
            "maxVotes": self.max_vote_count,
        }

    def to_model(self) -> Any:
        return EarlyStoppingRefereeModel(
            _t="EarlyStoppingReferee",
            threshold=self.threshold,
            maxVotes=self.max_vote_count,
        )



from abc import ABC, abstractmethod
from typing import Any

from rapidata.api_client.models.simple_workflow_model import SimpleWorkflowModel
from rapidata.rapidata_client.feature_flags import FeatureFlags
from rapidata.rapidata_client.referee.base_referee import Referee


class Workflow(ABC):
    
    def __init__(self, type: str):
        self._type = type
        self._target_country_codes: list[str] = []
        self._feature_flags: FeatureFlags = FeatureFlags()

    def to_dict(self) -> dict[str, Any]:
        return {
            "_t": self._type,
            "referee": self._referee.to_dict(),
            "targetCountryCodes": self._target_country_codes,
            "featureFlags": self._feature_flags.to_list(),
        }
    
    @abstractmethod
    def to_model(self) -> SimpleWorkflowModel:
        pass

    def referee(self, referee: Referee):
        self._referee = referee
        return self
    
    def target_country_codes(self, target_country_codes: list[str]):
        self._target_country_codes = target_country_codes
        return self
    
    def feature_flags(self, feature_flags: FeatureFlags):
        self._feature_flags = feature_flags
        return self



# flake8: noqa

# import apis into api package
from rapidata.api_client.api.coco_api import CocoApi
from rapidata.api_client.api.datapoint_api import DatapointApi
from rapidata.api_client.api.dataset_api import DatasetApi
from rapidata.api_client.api.identity_api import IdentityApi
from rapidata.api_client.api.newsletter_api import NewsletterApi
from rapidata.api_client.api.order_api import OrderApi
from rapidata.api_client.api.rapid_api import RapidApi
from rapidata.api_client.api.validation_api import ValidationApi




from rapidata.api_client.models.aggregator_type import AggregatorType
from rapidata.api_client.models.create_order_model import CreateOrderModel
from rapidata.api_client.models.create_order_model_referee import (
    CreateOrderModelReferee,
)
from rapidata.api_client.models.create_order_model_selections_inner import (
    CreateOrderModelSelectionsInner,
)
from rapidata.api_client.models.create_order_model_user_filters_inner import (
    CreateOrderModelUserFiltersInner,
)
from rapidata.api_client.models.create_order_model_workflow import (
    CreateOrderModelWorkflow,
)
from rapidata.api_client.models.country_user_filter_model import CountryUserFilterModel
from rapidata.rapidata_client.feature_flags import FeatureFlags
from rapidata.rapidata_client.metadata.base_metadata import Metadata
from rapidata.rapidata_client.dataset.rapidata_dataset import RapidataDataset
from rapidata.rapidata_client.referee.naive_referee import NaiveReferee
from rapidata.rapidata_client.selection.base_selection import Selection
from rapidata.rapidata_client.workflow import Workflow
from rapidata.rapidata_client.order.rapidata_order import RapidataOrder
from rapidata.rapidata_client.referee import Referee
from rapidata.service.openapi_service import OpenAPIService


class RapidataOrderBuilder:
    """Builder object for creating Rapidata orders.

    Use the fluent interface to set the desired configuration. Add a workflow to the order using `.workflow()` and finally call `.create()` to create the order.

    Args:
        openapi_service (OpenAPIService): The OpenAPIService instance.
        name (str): The name of the order.
    """

    def __init__(
        self,
        openapi_service: OpenAPIService,
        name: str,
    ):
        self._name = name
        self._openapi_service = openapi_service
        self._workflow: Workflow | None = None
        self._referee: Referee | None = None
        self._media_paths: list[str | list[str]] = []
        self._metadata: list[Metadata] | None = None
        self._aggregator: AggregatorType | None = None
        self._validation_set_id: str | None = None
        self._feature_flags: FeatureFlags | None = None
        self._country_codes: list[str] | None = None
        self._selections: list[Selection] = []
        self._rapids_per_bag: int = 2

    def _to_model(self) -> CreateOrderModel:
        if self._workflow is None:
            raise ValueError("You must provide a blueprint to create an order.")

        if self._referee is None:
            print("No referee provided, using default NaiveReferee.")
            self._referee = NaiveReferee()
        if self._country_codes is None:
            country_filter = None
        else:
            country_filter = CountryUserFilterModel(
                _t="CountryFilter", countries=self._country_codes
            )

        return CreateOrderModel(
            orderName=self._name,
            workflow=CreateOrderModelWorkflow(self._workflow.to_model()),
            userFilters=(
                [CreateOrderModelUserFiltersInner(country_filter)]
                if country_filter
                else []
            ),
            referee=CreateOrderModelReferee(self._referee.to_model()),
            validationSetId=self._validation_set_id,
            featureFlags=(
                self._feature_flags.to_list()
                if self._feature_flags is not None
                else None
            ),
            selections=[
                CreateOrderModelSelectionsInner(selection.to_model())
                for selection in self._selections
            ],
        )

    def create(self, submit=True, max_workers=10) -> RapidataOrder:
        """Actually makes the API calls to create the order based on how the order builder was configured.

        Args:
            submit (bool, optional): Whether to submit the order. Defaults to True.

        Returns:
            RapidataOrder: The created RapidataOrder instance.

        Raises:
            ValueError: If no workflow is provided.
        """
        order_model = self._to_model()

        result = self._openapi_service.order_api.order_create_post(
            create_order_model=order_model
        )

        self.order_id = result.order_id
        self._dataset = RapidataDataset(result.dataset_id, self._openapi_service)
        order = RapidataOrder(
            order_id=self.order_id,
            dataset=self._dataset,
            openapi_service=self._openapi_service,
        )

        order.dataset.add_media_from_paths(self._media_paths, self._metadata, max_workers)

        if submit:
            order.submit()

        return order

    def workflow(self, workflow: Workflow):
        """Set the workflow for the order.

        Args:
            workflow (Workflow): The workflow to be set.

        Returns:
            RapidataOrderBuilder: The updated RapidataOrderBuilder instance.
        """
        self._workflow = workflow
        return self

    def referee(self, referee: Referee):
        """Set the referee for the order.

        Args:
            referee (Referee): The referee to be set.

        Returns:
            RapidataOrderBuilder: The updated RapidataOrderBuilder instance.
        """
        self._referee = referee
        return self

    def media(
        self,
        media_paths: list[str | list[str]],
        metadata: list[Metadata] | None = None,
    ):
        """Set the media assets for the order.

        Args:
            media_paths (list[str | list[str]]): The paths of the media assets to be set.
            metadata (list[Metadata] | None, optional): Metadata for the media assets. Defaults to None.

        Returns:
            RapidataOrderBuilder: The updated RapidataOrderBuilder instance.
        """
        self._media_paths = media_paths
        self._metadata = metadata
        return self

    def feature_flags(self, feature_flags: FeatureFlags):
        """Set the feature flags for the order.

        Args:
            feature_flags (FeatureFlags): The feature flags to be set.

        Returns:
            RapidataOrderBuilder: The updated RapidataOrderBuilder instance.
        """
        self._feature_flags = feature_flags
        return self

    def country_filter(self, country_codes: list[str]):
        """Set the target country codes for the order.

        Args:
            country_codes (list[str]): The country codes to be set.

        Returns:
            RapidataOrderBuilder: The updated RapidataOrderBuilder instance.
        """
        self._country_codes = country_codes
        return self

    def aggregator(self, aggregator: AggregatorType):
        """Set the aggregator for the order.

        Args:
            aggregator (AggregatorType): The aggregator to be set.

        Returns:
            RapidataOrderBuilder: The updated RapidataOrderBuilder instance.
        """
        self._aggregator = aggregator
        return self

    def validation_set_id(self, validation_set_id: str):
        """Set the validation set for the order.

        Args:
            validation_set_id (str): The validation set ID to be set.

        Returns:
            RapidataOrderBuilder: The updated RapidataOrderBuilder instance.
        """
        self._validation_set_id = validation_set_id
        return self

    def rapids_per_bag(self, amount: int):
        """Defines the number of tasks a user sees in a single session.

        Args:
            amount (int): The number of tasks a user sees in a single session.

        Returns:
            RapidataOrderBuilder: The updated RapidataOrderBuilder instance.

        Raises:
            NotImplementedError: This method is not implemented yet.
        """
        raise NotImplementedError("Not implemented yet.")

    def selections(self, selections: list[Selection]):
        """Set the selections for the order.

        Args:
            selections (list[Selection]): The selections to be set.

        Returns:
            RapidataOrderBuilder: The updated RapidataOrderBuilder instance.
        """
        self._selections = selections
        return self





from rapidata.rapidata_client.selection.base_selection import Selection
from rapidata.api_client.models.demographic_selection import DemographicSelection as DemographicSelectionModel


class DemographicSelection(Selection):
    """Demographic selection class."""

    def __init__(self, rapid_id: str):
        self.rapid_id = rapid_id

    def to_model(self):
        return DemographicSelectionModel(_t="DemographicSelection", rapidId=self.rapid_id)